#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <portbits.h>
#include "8x14_horizontal_MSB_1.h"
#include "global.h"
#include "lcd.h"


unsigned char XStart, YCnt;
unsigned char * const mem=(unsigned char * const)DDRAM;
unsigned char cgram[8][8];



void lcd_init(void)
{	cli();


	MCUCR=(1<<SRE);								// Enable XMEM
	OCR1A=10;
	OCR1B=128;
	TCCR1A=(1<<COM1B1)|(1<<COM1B0)|(1<<WGM10);	// Enable PWM (Kontrast)
	TCCR1B=1;									// 8bit PWM

	OCR0=RELOAD;
	TCCR0=(1<<WGM01)|2;							// 2MHz Timer Takt, CTC
	TIMSK=(1<<OCIE0);

    OCR1B=10;                                       //KONTRAST VOR EINSTELLEN

	sei();
	

	
	
	lcd_clear();
	

	

	
	

	lcd_writechar(60,230,'6');
	lcd_writechar(62,230,'4');
	lcd_writechar(64,230,'0');
	lcd_writechar(66,230,'x');
	lcd_writechar(68,230,'4');
	lcd_writechar(70,230,'8');
	lcd_writechar(72,230,'0');
	lcd_writechar(76,230,'L');
	lcd_writechar(78,230,'C');
	lcd_writechar(80,230,'D');
	lcd_writechar(84,230,'C');
	lcd_writechar(86,230,'o');
	lcd_writechar(88,230,'n');
	lcd_writechar(90,230,'t');
	lcd_writechar(92,230,'r');
	lcd_writechar(94,230,'o');
	lcd_writechar(96,230,'l');
	lcd_writechar(98,230,'l');
	lcd_writechar(100,230,'e');
	lcd_writechar(102,230,'r');
	lcd_writechar(68,250,0xB8);
	lcd_writechar(72,250,'b');
	lcd_writechar(74,250,'y');
	lcd_writechar(78,250,'B');
	lcd_writechar(80,250,'e');
	lcd_writechar(82,250,'n');
	lcd_writechar(84,250,'e');
	lcd_writechar(86,250,'d');
	lcd_writechar(88,250,'i');
	lcd_writechar(90,250,'k');
	lcd_writechar(92,250,'t');
	
	




	wdt_enable(WDTO_250MS);
	ENABLE_VLCD=1;	
	ENABLE_Backlight=1;
}




void lcd_clear(void)
{

 unsigned short x,y;

 for(y=0; y<YSIZE; y++)
  for(x=0; x<XSIZE; x++) lcd_setpixel(x,y,0);


}



void lcd_clearfill(void)
{

 unsigned short x,y;


 for(y=0; y<YSIZE; y++)
  for(x=0; x<XSIZE; x++) lcd_setpixel(x,y,255);


}

void bresenham_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{	signed short dx,dy,x,y;
	signed char sdx,sdy;
	unsigned short i,px,py;

	dx=(signed int)x2-x1;      /* the horizontal distance of the line */
	dy=(signed int)y2-y1;      /* the vertical distance of the line */
	if (dx==0)
		sdx=0;
	if (dx>0)
		sdx=1;
	else
	{  	sdx=-1;
		dx=-dx;
	}
	if (dy==0)
		sdy=0;
	if (dy>0)
		sdy=1;
	else
	{	sdy=-1;
		dy=-dy;
	}
	x=dx*2;
	y=dy*2;
	px=x1;
	py=y1;
	lcd_setpixel (px,py,color);

	if (dx>=dy)			 // the line is more horizontal than vertical
	{	for(i=0;i<dx;i++)
		{	y+=dy;
			if (y>=dx)
			{	y-=dx;
				py+=sdy;
			}
			px+=sdx;
			lcd_setpixel (px,py,color);
		}
	}
	else				 // the line is more vertical than horizontal
	{	for(i=0;i<dy;i++)
		{	x+=dx;
			if (x>=dy)
			{	x-=dy;
				px+=sdx;
			}
			py+=sdy;
			lcd_setpixel (px,py,color);
		}
	}
}

void lcd_line (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color)
{   unsigned short temp;

	if (y1==y2)
	{	if (x1>x2)
		{	temp=x1;
			x1=x2;
			x2=temp;
		}
		if ((x2-x1)>=8)		// fast horizontal line using Byte Write
		{	unsigned char ctemp;
			if (color)
				ctemp=255;
			else
				ctemp=0;
			for (temp=x1; temp<((x1+3)&0xFFFC); temp++)
				lcd_setpixel (temp,y1,color);
			for (; temp<((x2-4)&0xFFFC); temp+=8)
				lcd_writebyte (temp/4, y1, ctemp,0);
			for (; temp<=x2; temp++)
				lcd_setpixel (temp,y1,color);
		}
		else
		{	for (temp=x1; temp<=x2; temp++)
				lcd_setpixel (temp,y1,color);
		}
	}
	else if (x1==x2)
	{	if (y1>y2)
		{	temp=y1;
			y1=y2;
			y2=temp;
		}
		for (temp=y1; temp<=y2; temp++)
		{	lcd_setpixel (x1,temp,color);
		}
	}
	else
		bresenham_line (x1, y1, x2, y2, color);
}
void lcd_circle(unsigned short xm, unsigned short ym, unsigned char r, unsigned char color1, unsigned char color2)
{	unsigned short x;
	unsigned char y=r;
	signed short p;
	if ((r>xm)||(r>ym)||(r<1))
		return;
	y=r;
	p=(3-(2*r));
	for (x=0; x<=y; x++)
	{	lcd_line(xm-x,ym+y,xm+x,ym+y,color1);
		lcd_line(xm-x,ym-y,xm+x,ym-y,color1);
		lcd_line(xm-y,ym+x,xm+y,ym+x,color1);
		lcd_line(xm-y,ym-x,xm+y,ym-x,color1);
		if(p<0)
			p+=((4*x)+6);
		else
		{	y--;
			p+=((4*(x-y))+10);
		}
	}
	y=r;
	p=(3-(2*r));
	for (x=0; x<=y; x++)
	{	lcd_setpixel((xm+x),(ym+y),color2);
   		lcd_setpixel((xm+y),(ym+x),color2);
   		lcd_setpixel((xm+y),(ym-x),color2);
   		lcd_setpixel((xm+x),(ym-y),color2);
   		lcd_setpixel((xm-x),(ym-y),color2);
   		lcd_setpixel((xm-y),(ym-x),color2);
   		lcd_setpixel((xm-y),(ym+x),color2);
   		lcd_setpixel((xm-x),(ym+y),color2);
		if(p<0)
			p+=((4*x)+6);
		else
		{	y--;
			p+=((4*(x-y))+10);
		}
	}
}




void lcd_block (unsigned short x1, unsigned short y1, unsigned short x2, unsigned short y2, unsigned char color1, unsigned char color2)
{	unsigned short y;
	if (x1>x2)
	{	y=x1;
		x1=x2;
		x2=y;
	}
	if (y1>y2)
	{	y=y1;
		y1=y2;
		y2=y;
	}
	if (x2>x1+1)
	{	for (y=y1+1; y<y2; y++)
			lcd_line(x1+1,y,x2-1,y,color1);
	}
	lcd_line(x1,y1,x2,y1,color2);
	lcd_line(x2,y1,x2,y2,color2);
	lcd_line(x2,y2,x1,y2,color2);
	lcd_line(x1,y2,x1,y1,color2);
}

void lcd_writecgram(unsigned char x, unsigned short y, unsigned char nr)
{	nr&=7;
	lcd_writebyte(x,y+0,cgram[nr][0],0);
	lcd_writebyte(x,y+1,cgram[nr][0],0);
	lcd_writebyte(x,y+2,cgram[nr][1],0);
	lcd_writebyte(x,y+3,cgram[nr][2],0);
	lcd_writebyte(x,y+4,cgram[nr][2],0);
	lcd_writebyte(x,y+5,cgram[nr][3],0);
	lcd_writebyte(x,y+6,cgram[nr][4],0);
	lcd_writebyte(x,y+7,cgram[nr][5],0);
	lcd_writebyte(x,y+8,cgram[nr][5],0);
	lcd_writebyte(x,y+9,cgram[nr][6],0);
	lcd_writebyte(x,y+10,cgram[nr][7],0);
	lcd_writebyte(x,y+11,cgram[nr][7],0);
}

